/*
 * FileCellRenderer.java - renders list and tree cells for QuickFile.
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.File;
import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.border.*;

import org.gjt.sp.jedit.*;

public final class FileCellRenderer implements javax.swing.ListCellRenderer,
                                          javax.swing.tree.TreeCellRenderer {

  private JLabel listCellRenderer = null;
  private JLabel treeCellRenderer = null;

  private Font normalFont;
  private Font openedFont;

  private Icon fileIcon;
  private Icon dirClosedIcon;
  private Icon dirOpenIcon;

  private Color treeSelectionForeground;
  private Color treeNoSelectionForeground;
  private Color treeSelectionBackground;
  private Color treeNoSelectionBackground;

  public FileCellRenderer() {
    Font f = UIManager.getFont("Tree.font");
    normalFont = new Font(f.getName(), Font.PLAIN, f.getSize());
    openedFont = new Font(f.getName(), Font.BOLD , f.getSize());

    treeSelectionForeground = UIManager.getColor("Tree.selectionForeground");
    treeNoSelectionForeground = UIManager.getColor("Tree.textForeground");
    treeSelectionBackground = UIManager.getColor("Tree.selectionBackground");
    treeNoSelectionBackground = UIManager.getColor("Tree.textBackground");

    fileIcon      = UIManager.getIcon("Tree.leafIcon");
    dirClosedIcon = UIManager.getIcon("Tree.closedIcon");
    dirOpenIcon   = UIManager.getIcon("Tree.openIcon");
  }

  protected Font getFontForFile(File file, boolean opened) {
    return opened? openedFont : normalFont;
  }

  protected Icon getIconForFile(File file, boolean expanded) {
    if(file.isDirectory())
      return expanded ? dirOpenIcon : dirClosedIcon;
    else
      return fileIcon;
  }

  public Component getListCellRendererComponent(JList list, Object value,
                                                int index, boolean sel,
                                                boolean focus) {
    if(listCellRenderer == null) {
      listCellRenderer = new JLabel();
      listCellRenderer.setOpaque(true);
    }

    File file = ((FileListModel.File)value).getFile();
    boolean opened = (jEdit.getBuffer(
        MiscUtilities.constructPath(null, file.getPath())) != null);

    if(sel) {
      listCellRenderer.setBackground(list.getSelectionBackground());
      listCellRenderer.setForeground(list.getSelectionForeground());
    }
    else {
      listCellRenderer.setBackground(list.getBackground());
      listCellRenderer.setForeground(list.getForeground());
    }
    listCellRenderer.setFont(getFontForFile(file, opened));
    listCellRenderer.setIcon(getIconForFile(file, sel));
    listCellRenderer.setText(value.toString());
    listCellRenderer.setEnabled(list.isEnabled());

    return listCellRenderer;
  }

  public Component getTreeCellRendererComponent(JTree tree, Object value,
                                                boolean sel, boolean expanded,
                                                boolean leaf, int row,
                                                boolean focus) {
    if(treeCellRenderer == null) {
      treeCellRenderer = new JLabel() {
        public Dimension getPreferredSize() {
          // this prevents the "..." from showing up
          Dimension d = super.getPreferredSize();
          if(d != null) d = new Dimension(d.width + 7, d.height);
          return d;
        }
      };
      treeCellRenderer.setOpaque(true);
    }

    File file = ((FileTreeNode)value).getFile();
    boolean opened = (jEdit.getBuffer(
        MiscUtilities.constructPath(null, file.getPath())) != null);

    if(sel) {
      treeCellRenderer.setBackground(treeSelectionBackground);
      treeCellRenderer.setForeground(treeSelectionForeground);
    }
    else {
      treeCellRenderer.setBackground(treeNoSelectionBackground);
      treeCellRenderer.setForeground(treeNoSelectionForeground);
    }
    treeCellRenderer.setFont(getFontForFile(file, opened));
    treeCellRenderer.setIcon(getIconForFile(file, expanded));
    treeCellRenderer.setText(value.toString());
    treeCellRenderer.setEnabled(tree.isEnabled());

    return treeCellRenderer;
  }
}

