/*
 * FileTreeNode.java - used to display directory tree and files in a JTree
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.util.*;
import java.io.File;
import javax.swing.tree.TreeNode;
import javax.swing.filechooser.FileFilter;
import org.gjt.sp.jedit.MiscUtilities;

/**
 * A node that allows easy creation of a JTree that displays
 * a file system rooted at an arbitrary directory.  To create
 * such a tree, use code similar to the following:
 * <PRE>
 *     JTree fileTree = new JTree(new FileTreeNode(rootDirectory));
 * </PRE>
 *
 * @author Jason Ginchereau
 */
public class FileTreeNode implements TreeNode/*, Comparable*/ {

  static   boolean sort;
  static   boolean caseSensitive;


  private   FileTreeNode parent;
  private   File         file;
  private   FileFilter   filter;

  private   boolean      directory;
  protected Vector       children;

  public FileTreeNode(File file) {
    this(null, file, null);
  }

  /**
   * Creates a node that only returns children which pass the filter.
   */
  public FileTreeNode(File file, FileFilter filter) {
    this(null, file, filter);
  }

  protected FileTreeNode(FileTreeNode parent, File file, FileFilter filter) {
    this.parent    = parent;
    this.file      = file;
    this.filter    = filter;
    this.children  = null;
    this.directory = file != null && file.isDirectory();
  }

  public TreeNode getParent() {
    return parent;
  }

  public File getFile() {
    return file;
  }

  public int getChildCount() {
    return getChildren().size();
  }

  public TreeNode getChildAt(int childIndex) {
    return (TreeNode) getChildren().elementAt(childIndex);
  }

  public Enumeration children() {
    return getChildren().elements();
  }

  public int getIndex(TreeNode node) {
    return getChildren().indexOf(node);
  }

  public boolean getAllowsChildren() {
    return directory;
  }

  public boolean isLeaf() {
    return !directory;
  }

  public final void refresh(FileFilter filter,
                            javax.swing.tree.DefaultTreeModel model) {
    this.filter = filter;

    if(children == null || !directory) return;

    synchronized(this) {
      Vector newChildren = getChildren(this);

      // first, remove any non-existant children
      Vector removedIndices = new Vector();
      Object o;
      int i, j;
      for(i = 0; i < children.size(); i++) {
        o = children.elementAt(i);
        if(!newChildren.contains(o)) {
          removedIndices.addElement(new Integer(i));
        }
      }
      if(removedIndices.size() > 0) {
        int[] indices = new int[removedIndices.size()];
        Object[] c = new Object[removedIndices.size()];
        for(i = 0; i < c.length; i++) {
          indices[i] = ((Integer)removedIndices.elementAt(i)).intValue();
          c[i] = children.elementAt(indices[i] - i);
          children.removeElementAt(indices[i] - i);
        }
        model.nodesWereRemoved(this, indices, c);
      }

      // next, add any new children
      Vector addedIndices = new Vector();
      for(i = 0; i < newChildren.size(); i++) {
        o = newChildren.elementAt(i);
        if(!children.contains(o)) {
          if(sort) {
            File of = ((FileTreeNode)o).getFile();
            boolean odir = of.isDirectory();
            for(j = children.size(); j > 0; j--) {
              //if(children.elementAt(j-1) < o) break;
              File cf = ((FileTreeNode)children.elementAt(j-1)).getFile();
              boolean cdir = cf.isDirectory();
              if(!odir && cdir) break;
              else if(odir == cdir) {
                if(caseSensitive) {
                  if(cf.getName().compareTo(of.getName()) < 0) break;
                } else {
                  if(cf.getName().toLowerCase().compareTo(of.getName()
                     .toLowerCase()) < 0) break;
                }
              }
            }
          }
          else {
            j = children.size();
          }
          addedIndices.addElement(new Integer(j));
          children.insertElementAt(o, j);
        }
      }
      if(addedIndices.size() > 0) {
        int[] indices = new int[addedIndices.size()];
        for(i = 0; i < indices.length; i++) {
          indices[i] = ((Integer)addedIndices.elementAt(i)).intValue();
        }
        model.nodesWereInserted(this, indices);
      }
    }

    // finaly, refresh all children
    for(Enumeration e = children(); e.hasMoreElements(); )
      ((FileTreeNode)e.nextElement()).refresh(filter, model);
  }

  private Vector getChildren() {
    if(children == null) {
      // double-checked locking pattern only uses
      // synchronization when necessary
      synchronized(this) {
        if(children == null) {
          if(directory) children = getChildren(this);
          else children = new Vector();
        }
      }
    }
    return children;
  }

  private static Vector getChildren(FileTreeNode n) {
    Vector children = new Vector();

    String[] fl = n.file.list();
    if(sort) sortStrings(fl, caseSensitive);
    for(int i = 0; i < fl.length; i++) {
      File f = new File(n.file, fl[i]);
      if(n.filter == null || n.filter.accept(f)) {
        children.addElement(new FileTreeNode(n, f, n.filter));
      }
    }

    // put directories first (but keep ordering)
    int dirIndex = -1;
    for(int i = 0; i < children.size(); i++) {
      FileTreeNode c = (FileTreeNode) children.elementAt(i);
      if(c.directory && i != ++dirIndex) {
        children.removeElementAt(i--);
        children.insertElementAt(c, dirIndex);
      }
    }

    return children;
  }

  // java.util.Collections is not available in Java 1.1
  static void sortStrings(String[] array, boolean caseSensitive) {
    if(array.length == 0) return;

    MiscUtilities.Compare comp;
    if(caseSensitive)
      comp = new MiscUtilities.StringCompare();
    else
      comp = new MiscUtilities.StringICaseCompare();

    MiscUtilities.quicksort(array,comp);
  }

  public final boolean equals(Object o) {
    return o instanceof FileTreeNode &&
      (file == null && ((FileTreeNode)o).file == null ||
       (file != null && ((FileTreeNode)o).file != null &&
        file.equals(((FileTreeNode)o).file)));
  }

  public String toString() {
    return file.getName();
  }

  public static class Parent extends FileTreeNode {
    public Parent(File parentDir, File dir, FileFilter filter) {
      super(parentDir);
      children = new Vector();
      if(parentDir != null) {
        children.addElement(new Parent(this, parentDir));
      }
      children.addElement(new FileTreeNode(this, dir, filter));
    }

    protected Parent(Parent parent, File parentDir) {
      super(parent, parentDir, null);
      children = new Vector();
    }

    public boolean getAllowsChildren() {
      return this.getChildCount() > 0;
    }

    public String toString() {
      return "..";
    }
  }
}
