/*
 * QuickFileExplorerView.java - 
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;
import org.gjt.sp.jedit.jEdit;

/**
 *
 * @author Jason Ginchereau
 */
public class QuickFileExplorerView extends QuickFileView
                                   implements TreeSelectionListener,
                                              ListSelectionListener,
                                              MouseListener {

  protected JTree        dirTree;
  protected FileTreeNode dirTreeRoot;
  protected JList        fileList;
  private   JScrollPane  dirTreeScroller;
  private   JScrollPane  fileListScroller;
  private   JSplitPane   listSplitter;

  private static FileListModel blankModel
      = new FileListModel(null, null, false);

  private static FileFilter dirFilter = new FileFilter() {
    public boolean accept(File f) { return f.isDirectory(); }
    public String getDescription() { return "directories"; }
  };

  private static FileCellRenderer renderer = new FileCellRenderer();

  public QuickFileExplorerView(File dir) {
    super(dir);

    updateSorting();

    String parent = null;
    if(jEdit.getBooleanProperty("quickfile.showparent"))
      parent = dir.getParent();
    if(parent != null) {
      FileTreeNode.Parent hiddenRoot = new FileTreeNode.Parent(
          new File(parent), dir, dirFilter);
      dirTreeRoot = (FileTreeNode)
          hiddenRoot.getChildAt(hiddenRoot.getChildCount()-1);
      dirTree = new HelpfulJTree(hiddenRoot);
    }
    else {
      dirTreeRoot = new FileTreeNode(dir, dirFilter);
      dirTree = new HelpfulJTree(dirTreeRoot);
    }

    dirTree.setEditable(false);
    dirTree.setScrollsOnExpand(
        !jEdit.getBooleanProperty("quickfile.dc-chroot"));
    dirTree.setShowsRootHandles(true);
    dirTree.setRootVisible(false);
    dirTree.getSelectionModel()
           .setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
    dirTree.setCellRenderer(renderer);
    dirTree.addTreeSelectionListener(this);
    dirTree.addMouseListener(this);
    dirTree.setMinimumSize(new Dimension(0, 0));

    fileList = new HelpfulJList(blankModel);
    fileList.setCellRenderer(renderer);
    fileList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    fileList.addListSelectionListener(this);
    fileList.addMouseListener(this);
    fileList.setMinimumSize(new Dimension(0, 0));

    dirTreeScroller = new JScrollPane(dirTree);
    fileListScroller = new JScrollPane(fileList);

    listSplitter = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                                  dirTreeScroller, fileListScroller);
    listSplitter.setBorder(null);
    listSplitter.setOneTouchExpandable(false);
    listSplitter.setContinuousLayout(true);
    QuickFilePlugin.setResizeWeight(listSplitter, 0.5);

    if(parent != null) {
      dirTree.expandRow(1);
      dirTree.setSelectionRow(1);
    }

    setLayout(new BorderLayout());
    add(BorderLayout.CENTER, listSplitter);
  }

  private void updateSorting() {
    FileTreeNode.sort
        = jEdit.getBooleanProperty("quickfile.sort");
    FileTreeNode.caseSensitive
        = jEdit.getBooleanProperty("quickfile.sort.cs");
    FileListModel.sort = FileTreeNode.sort;
    FileListModel.caseSensitive = FileTreeNode.caseSensitive;
  }

  private boolean splitterInitialized = false;
  public void doLayout() {
    super.doLayout();
    if(!splitterInitialized) {
      listSplitter.setDividerLocation(0.33);
      splitterInitialized = true;
    }
  }

  public File getSelectedDirectory() {
    TreePath path = dirTree.getSelectionPath();
    if(path == null) return null;
    FileTreeNode n = (FileTreeNode) path.getLastPathComponent();
    return n.getFile();
  }

  public File getSelectedFile() {
    FileListModel.File mf = (FileListModel.File) fileList.getSelectedValue();
    return mf != null? mf.getFile() : null;
  }

  /**
   * Refreshes this view by checking for newly created or deleted files.
   * Also re-applies the filter.
   */
  public void refresh() {
    updateSorting();

    dirTreeRoot.refresh(dirFilter, (DefaultTreeModel)dirTree.getModel());
    ((FileListModel)fileList.getModel()).refresh(filter);
  }

  // TreeSelectionListener interface:
  public void valueChanged(TreeSelectionEvent evt) {
    if(evt.isAddedPath()) {
      FileTreeNode node = (FileTreeNode) evt.getPath().getLastPathComponent();
      File file = (node != null? node.getFile() : null);
      if(file != null)
        fileList.setModel(new FileListModel(file, filter, false));
      else
        fileList.setModel(blankModel);
      fireFileSelected(file);
    }
  }

  // ListSelectionListener interface:
  public void valueChanged(ListSelectionEvent evt) {
    FileListModel.File mf = (FileListModel.File)fileList.getSelectedValue();
    fireFileSelected(mf != null? mf.getFile() : null);
  }

  // MouseListener interface:
  public void mouseClicked(MouseEvent evt) {
    if((evt.getModifiers() & MouseEvent.BUTTON1_MASK) != 0 &&
       evt.getClickCount() == 2) {
      if(evt.getSource() == dirTree) {
        TreePath path = dirTree.getPathForLocation(evt.getX(), evt.getY());
        FileTreeNode node = path != null?
            (FileTreeNode) path.getLastPathComponent() : null;
        if(node != null) fireFileActivated(node.getFile());
      }
      else if(evt.getSource() == fileList) {
        FileListModel.File mf;
        mf = (FileListModel.File)fileList.getSelectedValue();
        if(mf != null) fireFileActivated(mf.getFile());
      }
    }
    else if(evt.isPopupTrigger()) doPopup(evt);
  }
  public void mousePressed(MouseEvent evt)  {
    if(evt.getSource() == dirTree) {
      TreePath path = dirTree.getPathForLocation(evt.getX(), evt.getY());
      if(path != null && !path.equals(dirTree.getSelectionPath()))
        dirTree.setSelectionPath(path);
    }
    else if(evt.getSource() == fileList) {
      int index = fileList.locationToIndex(evt.getPoint());
      if(index >= 0) {
        if(fileList.getSelectedIndex() != index)
          fileList.setSelectedIndex(index);
      }
    }
    if(evt.isPopupTrigger()) doPopup(evt);
  }
  public void mouseReleased(MouseEvent evt) {
    if(evt.isPopupTrigger()) doPopup(evt);
  }
  public void mouseEntered(MouseEvent evt)  { }
  public void mouseExited(MouseEvent evt)   { }
  
  private void doPopup(MouseEvent evt) {
    if(evt.getSource() == dirTree) {
      TreePath path = dirTree.getPathForLocation(evt.getX(), evt.getY());
      FileTreeNode node = path != null?
          (FileTreeNode) path.getLastPathComponent() : null;
      if(node != null)
        showFilePopup(node.getFile(), (Component) evt.getSource(), evt.getPoint());
    }
    else if(evt.getSource() == fileList) {
      FileListModel.File mf;
      mf = (FileListModel.File)fileList.getSelectedValue();
      if(mf != null)
        showFilePopup(mf.getFile(), (Component) evt.getSource(), evt.getPoint());
    }
  }
}
