/*
 * QuickFileListView.java - 
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.gjt.sp.jedit.jEdit;

/**
 *
 * @author Jason Ginchereau
 */
public class QuickFileListView extends QuickFileView
                               implements ListSelectionListener,
                                          MouseListener {

  protected JList        dirList;
  protected JList        fileList;
  private   JScrollPane  dirListScroller;
  private   JScrollPane  fileListScroller;
  private   JSplitPane   listSplitter;

  private static FileCellRenderer renderer = new FileCellRenderer();

  private static FileListModel blankModel
      = new FileListModel(null, null, false);

  public QuickFileListView(File dir) {
    super(dir);

    updateSorting();

    fileList = new HelpfulJList(blankModel);
    fileList.setCellRenderer(renderer);
    fileList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    fileList.addListSelectionListener(this);
    fileList.addMouseListener(this);
    fileList.setMinimumSize(new Dimension(0, 0));

    dirList = new JList(new FileListModel(dir, filter, true));
    dirList.setCellRenderer(renderer);
    dirList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    dirList.addListSelectionListener(this);
    dirList.addMouseListener(this);
    dirList.setMinimumSize(new Dimension(0, 0));
    if(dirList.getModel().getSize() > 0) dirList.setSelectedIndex(0);

    dirListScroller = new JScrollPane(dirList);
    fileListScroller = new JScrollPane(fileList);

    listSplitter = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                                  dirListScroller, fileListScroller);
    listSplitter.setBorder(null);
    listSplitter.setOneTouchExpandable(false);
    listSplitter.setContinuousLayout(true);
    QuickFilePlugin.setResizeWeight(listSplitter, 0.5);

    setLayout(new BorderLayout());
    add(BorderLayout.CENTER, listSplitter);
  }

  private void updateSorting() {
    FileListModel.sort
        = jEdit.getBooleanProperty("quickfile.sort");
    FileListModel.caseSensitive
        = jEdit.getBooleanProperty("quickfile.sort.cs");
  }

  private boolean splitterInitialized = false;
  public void doLayout() {
    super.doLayout();
    if(!splitterInitialized) {
      listSplitter.setDividerLocation(0.33);
      splitterInitialized = true;
    }
  }

  public File getSelectedDirectory() {
    FileListModel.File mf = (FileListModel.File) dirList.getSelectedValue();
    return mf != null? mf.getFile() : null;
  }

  public File getSelectedFile() {
    FileListModel.File mf = (FileListModel.File) fileList.getSelectedValue();
    return mf != null? mf.getFile() : null;
  }

  /**
   * Refreshes this view by checking for newly created or deleted files.
   * Also re-applies the filter.
   */
  public void refresh() {
    updateSorting();

    ((FileListModel)dirList.getModel()).refresh(filter);
    ((FileListModel)fileList.getModel()).refresh(filter);
  }

  // ListSelectionListener interface:
  public void valueChanged(ListSelectionEvent evt) {
    FileListModel.File mf;
    if(evt.getSource() == dirList) {
      mf = (FileListModel.File)dirList.getSelectedValue();
      if(mf != null)
        fileList.setModel(new FileListModel(mf.getFile(), filter, false));
      else
        fileList.setModel(blankModel);
    }
    else {
      mf = (FileListModel.File)fileList.getSelectedValue();
    }
    fireFileSelected(mf != null? mf.getFile() : null);
  }

  // MouseListener interface:
  public void mouseClicked(MouseEvent evt) {
    if((evt.getModifiers() & MouseEvent.BUTTON1_MASK) != 0 &&
       evt.getClickCount() == 2) {
      FileListModel.File mf;
      mf = (FileListModel.File)((JList) evt.getSource()).getSelectedValue();
      if(mf != null) fireFileActivated(mf.getFile());
    }
    else if(evt.isPopupTrigger()) doPopup(evt);
  }
  public void mousePressed(MouseEvent evt)  {
    JList list = (JList) evt.getSource();
    int index = list.locationToIndex(evt.getPoint());
    if(index >= 0) {
      if(list.getSelectedIndex() != index)
        list.setSelectedIndex(index);
    }
    if(evt.isPopupTrigger()) doPopup(evt);
  }
  public void mouseReleased(MouseEvent evt) {
    if(evt.isPopupTrigger()) doPopup(evt);
  }
  public void mouseEntered(MouseEvent evt)  { }
  public void mouseExited(MouseEvent evt)   { }
  
  private void doPopup(MouseEvent evt) {
    FileListModel.File mf;
    mf = (FileListModel.File)((JList) evt.getSource()).getSelectedValue();
    if(mf != null)
      showFilePopup(mf.getFile(), (Component) evt.getSource(), evt.getPoint());
  }
}
