/*
 * QuickFilePanel.java - 
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;

import org.gjt.sp.jedit.*;
import org.gjt.sp.jedit.gui.*;
import org.gjt.sp.jedit.msg.*;
import org.gjt.sp.util.Log;

/**
 *
 * @author Jason Ginchereau
 */
public class QuickFilePanel extends JPanel implements QuickFileViewListener,
                                                      EBComponent,
                                                      ActionListener {

  JPanel           toolPanel;
  HistoryTextField dirField;
  HistoryTextField extField;

  Hashtable        viewMap;
  QuickFileView    currentView;
  FileFilter       filter;

  AbstractButton   gotoDirButton;

  JFrame           frame;

  public QuickFilePanel(View v) {

    setLayout(new BorderLayout());
    toolPanel = createToolPanel();
    add(BorderLayout.NORTH, toolPanel);

    // default to current working directory
    try {
      dirField.setText(new File(".").getCanonicalPath());
    } catch(IOException ioex) { dirField.setText("."); }

    String start = jEdit.getProperty("quickfile.start");
    // start in parent directory of current buffer
    if(start.equals("buffer")) {
      dirField.setText(v.getBuffer().getFile().getParent());
    }
    // start in last directory selected in QuickFile
    else if(start.equals("last")) {
      HistoryModel hm = HistoryModel.getModel("quickfile.directory");
      if(hm.getSize() > 0) dirField.setText(hm.getItem(0));
    }
    dirField.addCurrentToHistory();

    viewMap = new Hashtable();

    filter = null;
    currentView = null;

    filter = new GlobFileFilter(extField.getText());
    setCurrentView(getView(dirField.getText()));
    refreshView(currentView);

    EditBus.addToBus(this);
  }
  
  // QuickFileViewListener interface:
  public void fileSelected(QuickFileView source, File file) {
    if(file != null && file.isDirectory()) {
      if(gotoDirButton != null)
        gotoDirButton.setEnabled(!file.equals(currentView.getRootDirectory()));
    }
    else {
      if(gotoDirButton != null) gotoDirButton.setEnabled(false);

      if(file != null) {
        Buffer buffer = jEdit.getBuffer(file.getPath());
        if(buffer != null) {
          View view = QuickFilePlugin.getViewForQuickFilePanel(this);
          view.setBuffer(buffer);
        }
      }
    }
  }

  public void fileActivated(QuickFileView source, File file) {
    View view = QuickFilePlugin.getViewForQuickFilePanel(this);
    if(file.isDirectory()) {
      if(jEdit.getBooleanProperty("quickfile.dc-chroot") &&
         !file.equals(currentView.getRootDirectory())) {
        dirField.setText(file.getPath());
        actionPerformed(new ActionEvent(dirField,
                                        ActionEvent.ACTION_PERFORMED, ""));
      }
    }
    else {
      Buffer buffer = jEdit.getBuffer(file.getPath());
      if(buffer == null) {
        jEdit.openFile(view, null, file.getPath(), false, false);
      }
      else if(jEdit.getBooleanProperty("quickfile.dc-close")) {
        jEdit.closeBuffer(view, buffer);
      }
    }
  }

  public void handleMessage(EBMessage msg) {
    if(msg instanceof BufferUpdate) {
      BufferUpdate bu = (BufferUpdate) msg;
      if(bu.getWhat() == BufferUpdate.CREATED ||
         bu.getWhat() == BufferUpdate.CLOSED) {
        bufferCreatedOrClosed();
      }
    }
    else if(msg instanceof PropertiesChanged) {
      propertiesChanged();
    }
  }

  void bufferCreatedOrClosed() {
    if(currentView != null) currentView.repaint();
  }

  void propertiesChanged() {
    JSplitPane sp = QuickFilePlugin.getSplitterForQuickFilePanel(this);
    if(sp != null) {
      JComponent c = (JComponent) sp.getLeftComponent();
      c.setMinimumSize(new Dimension(0, c.getMinimumSize().height));
    }

    remove(toolPanel);
    toolPanel = createToolPanel();
    add(BorderLayout.NORTH, toolPanel);

    if(currentView != null) {
      SetViewAction a = new SetViewAction(
          jEdit.getProperty("quickfile.view.default"));
      a.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED,""));
      refreshView(currentView);
    }

    revalidate();
    repaint();
  }

  // ActionListener interface:
  public void actionPerformed(ActionEvent evt) {
    HistoryTextField htf = (HistoryTextField)evt.getSource();
    htf.addCurrentToHistory();

    String item = htf.getText();
    if(item == null || item.equals("")) return;

    if(htf == dirField) {
      setCurrentView(getView(item));
      currentView.setFilter(filter);
      refreshView(currentView);
    }
    else if(htf == extField) {
      jEdit.setProperty("quickfile.filter", item);
      filter = new GlobFileFilter(item);
      if(currentView != null) {
        currentView.setFilter(filter);
        refreshView(currentView);
      }
    }
  }

  void setCurrentView(QuickFileView v) {
    if(currentView != null) {
      currentView.removeQuickFileViewListener(this);
      remove(currentView);
    }
    currentView = v;
    if(currentView != null) {
      add(BorderLayout.CENTER, currentView);
      currentView.addQuickFileViewListener(this);
      currentView.revalidate();
    }
    if(gotoDirButton != null) gotoDirButton.setEnabled(
        currentView != null && currentView.getSelectedDirectory() != null &&
        !currentView.getSelectedDirectory().equals(
         currentView.getRootDirectory()));
    repaint();
  }

  QuickFileView getView(String dir) {
    try {
      Class viewClass = null;

      if(currentView == null) {
        String defaultView = jEdit.getProperty("quickfile.view.default");
        String viewClassName
            = jEdit.getProperty("quickfile.view." + defaultView);
        viewClass = Class.forName(viewClassName);
      }
      else {
        viewClass = currentView.getClass();
      }
      return getView(dir, viewClass);
    }
    catch(Exception ex) {
      Log.log(Log.ERROR,this,"Exception creating QuickFileView");
      Log.log(Log.ERROR,this,ex);
    }
    return null;
  }

  QuickFileView getView(String dir, Class viewClass) {
    try {
      QuickFileView v = null;

      if(viewClass != null) synchronized(this) {
        Hashtable vh = (Hashtable)viewMap.get(viewClass);
        if(vh == null) {
          viewMap.put(viewClass, vh = new Hashtable());
        }
        else {
          v = (QuickFileView)vh.get(dir);
        }

        if(v == null) {
          Constructor c = viewClass.getConstructor(new Class[] { File.class });
          v = (QuickFileView)c.newInstance(new Object[] { new File(dir) });
          vh.put(dir, v);
        }
      }

      if(v != null) {
        v.setFilter(filter);
        //refreshView(v);
      }

      return v;
    }
    catch(InvocationTargetException itex) {
      Log.log(Log.ERROR,this,"Exception creating QuickFileView");
      if(itex.getTargetException() != null)
        Log.log(Log.ERROR,this,itex.getTargetException());
      else
        Log.log(Log.ERROR,this,itex);
    }
    catch(Exception ex) {
      Log.log(Log.ERROR,this,"Exception creating QuickFileView");
      Log.log(Log.ERROR,this,ex);
    }
    return null;
  }

  // Called before this QuickFilePanel is deleted.
  public void dispose() {
    EditBus.removeFromBus(this);
  }

  private JPanel createToolPanel() {
    boolean showRefresh
        = jEdit.getBooleanProperty("quickfile.showrefresh");
    boolean showSynchronize
        = jEdit.getBooleanProperty("quickfile.showsynchronize");
    boolean showFilter
        = jEdit.getBooleanProperty("quickfile.showfilter");

    if(dirField == null) {
      dirField = new HistoryTextField("quickfile.directory", true);
      dirField.addActionListener(this);
    }

    //AbstractButton treeViewButton  = loadCustomButton("quickfile.view.tree");
    //AbstractButton listViewButton  = loadCustomButton("quickfile.view.list");
    AbstractButton chooseDirButton = loadCustomButton("quickfile.choose-dir");
    if(jEdit.getBooleanProperty("quickfile.showgotodir")) {
      gotoDirButton = loadCustomButton("quickfile.goto-dir");
    }
    else {
      gotoDirButton = null;
    }
    AbstractButton refreshButton = null;
    if(showRefresh) refreshButton = loadCustomButton("quickfile.refresh");
    AbstractButton synchronizeButton = null;
    if(showSynchronize) synchronizeButton = loadCustomButton("quickfile.synchronize");

    Insets zeroMargin = new Insets(0, 0, 0, 0);
    //treeViewButton.setMargin(zeroMargin);
    //listViewButton.setMargin(zeroMargin);
    if(gotoDirButton != null) gotoDirButton.setMargin(zeroMargin);
    chooseDirButton.setMargin(zeroMargin);
    if(showRefresh) refreshButton.setMargin(zeroMargin);
	if (showSynchronize) synchronizeButton.setMargin(zeroMargin);
	
    if(extField == null) {
      extField = new HistoryTextField("quickfile.filter", true);
      //Font f = extField.getFont();
      //extField.setFont(new Font("Monospaced", f.getStyle(), f.getSize()));
      extField.addActionListener(this);
      extField.setMinimumSize(new Dimension(20,
                                            extField.getMinimumSize().height));

      HistoryModel hm = HistoryModel.getModel("quickfile.filter");
      if(hm.getSize() > 0) {
        extField.setText(hm.getItem(0));
      }
      else {
        extField.setText("*");
        extField.addCurrentToHistory();
      }
    }

    JComponent filterComponent;
    if(showFilter) {
      filterComponent = new JPanel();
      filterComponent.setLayout(new BoxLayout(filterComponent,
                                              BoxLayout.Y_AXIS));
      filterComponent.add(Box.createVerticalGlue());
      extField.setMaximumSize(new Dimension(Integer.MAX_VALUE,
                              extField.getPreferredSize().height));
      filterComponent.add(extField);
      filterComponent.add(Box.createVerticalGlue());
    }
    else {
      filterComponent = loadCustomButton("quickfile.filter");
    }

    JToolBar toolBar = new JToolBar();
    toolBar.setFloatable(false);
    //treeViewButton.setAlignmentY(0.0f);
    //listViewButton.setAlignmentY(0.0f);
    chooseDirButton.setAlignmentY(0.0f);
    if(gotoDirButton != null) gotoDirButton.setAlignmentY(0.0f);
    if(showRefresh) refreshButton.setAlignmentY(0.0f);
    if(showSynchronize) synchronizeButton.setAlignmentY(0.0f);
    filterComponent.setAlignmentY(0.0f);
    //toolBar.add(treeViewButton);
    //toolBar.add(listViewButton);
    //toolBar.addSeparator();
    toolBar.add(chooseDirButton);
    if(gotoDirButton != null) toolBar.add(gotoDirButton);
    if(showRefresh) toolBar.add(refreshButton);
    if(showSynchronize) toolBar.add(synchronizeButton);
    toolBar.add(filterComponent);

    JPanel toolPanel = new JPanel();
    toolPanel.setLayout(new BorderLayout());
    toolPanel.add(BorderLayout.CENTER, toolBar);
    toolPanel.add(BorderLayout.SOUTH, dirField);
    toolPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 3, 0));

    return toolPanel;
  }

  private AbstractButton loadCustomButton(String name) {
    String icon = jEdit.getProperty(name + ".icon");
    java.net.URL u = getClass().getResource(icon);

    /*
    // use a temporary icon to avoid a NPE
    jEdit.setProperty(name + ".icon", "Delete.gif");
    AbstractButton b = GUIUtilities.loadToolButton(name);
    jEdit.setProperty(name + ".icon", icon);
    if(u != null) b.setIcon(new ImageIcon(u));
    */
    
    ActionListener action = QuickFilePlugin.getAction(name);
    String toolTip = jEdit.getProperty(name.concat(".label"));
    AbstractButton b = new JButton(new ImageIcon(u));
    if(action != null) {
      b.addActionListener(action);
      b.setEnabled(true);
    }
    else {
      b.setEnabled(false);
    }
    b.setToolTipText(toolTip);
    b.setMargin(new Insets(0,0,0,0));
    b.setRequestFocusEnabled(false);
    
    return b;
  }

  public Dimension getMinimumSize() {
    Dimension min = super.getMinimumSize();
    min.width = 0;
    return min;
  }

  void refreshView(QuickFileView v) {
    if(v != null) {
      // Changing the cursor doesn't seem to work...
      //Cursor c = getCursor();
      //setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      v.refresh();
      //setCursor(c);
    }
  }

  public String paramString() {
    return dirField.getText();
  }

  static class SetViewAction implements ActionListener {
    private String type;

    public SetViewAction(String type) {
      this.type = type;
    }

    public void actionPerformed(ActionEvent evt) {
      QuickFilePanel qfp
          = QuickFilePlugin.getQuickFilePanelForView(
              QuickFilePlugin.getViewForComponent((Component)evt.getSource()));
      if(qfp == null) return;

      String viewClassName = jEdit.getProperty("quickfile.view." + type);
      Class viewClass;
      try {
        viewClass = Class.forName(viewClassName);
      } catch(Exception ex) {
        Log.log(Log.ERROR,this,"Exception creating QuickFileView");
        Log.log(Log.ERROR,this,ex);
        return;
      }

      if(qfp.currentView == null ||
         !qfp.currentView.getClass().equals(viewClass)) {
        qfp.setCurrentView(qfp.getView(qfp.dirField.getText(),
                                       viewClass));
      }
    }
  }

  static class ChooseDirAction implements ActionListener {
    public void actionPerformed(ActionEvent evt) {
      View v = QuickFilePlugin.getViewForComponent((Component)evt.getSource());
      QuickFilePanel qfp = QuickFilePlugin.getQuickFilePanelForView(v);
      if(qfp == null) return;

      String cd = qfp.dirField.getText();
      File currentDir = (cd != null? new File(cd) : new File("."));
      JFileChooser chooser = new JFileChooser(currentDir);
      chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
      if(chooser.showOpenDialog(v) == JFileChooser.APPROVE_OPTION) {
        File dir = chooser.getSelectedFile();
        qfp.dirField.setText(dir.getPath());
        qfp.dirField.addCurrentToHistory();
        qfp.actionPerformed(new ActionEvent(qfp.dirField,
                                            ActionEvent.ACTION_PERFORMED, ""));
      }
    }
  }

  static class GotoDirAction implements ActionListener {
    public void actionPerformed(ActionEvent evt) {
      View v = QuickFilePlugin.getViewForComponent((Component)evt.getSource());
      QuickFilePanel qfp = QuickFilePlugin.getQuickFilePanelForView(v);
      if(qfp == null) return;

      File dir = qfp.currentView.getSelectedDirectory();
      if(dir != null && dir.isDirectory()) {
        qfp.dirField.setText(dir.getPath());
        qfp.dirField.addCurrentToHistory();
        qfp.actionPerformed(new ActionEvent(qfp.dirField,
                                            ActionEvent.ACTION_PERFORMED, ""));
      }
    }
  }

  static class RefreshAction implements ActionListener {
    public void actionPerformed(ActionEvent evt) {
      QuickFilePanel qfp
          = QuickFilePlugin.getQuickFilePanelForView(
              QuickFilePlugin.getViewForComponent((Component)evt.getSource()));
      if(qfp == null) return;

      jEdit.setProperty("quickfile.filter", qfp.extField.getText());
      //qfp.filter = new ExtensionFileFilter(qfp.extField.getText());
      qfp.filter = new GlobFileFilter(qfp.extField.getText());
      if(qfp.currentView != null) {
        qfp.currentView.setFilter(qfp.filter);
        qfp.refreshView(qfp.currentView);
      }
    }
  }

  static class SetFilterAction implements ActionListener {
    public void actionPerformed(ActionEvent evt) {
      View v = QuickFilePlugin.getViewForComponent((Component)evt.getSource());
      QuickFilePanel qfp = QuickFilePlugin.getQuickFilePanelForView(v);
      if(qfp == null) return;

      JLabel label1 = new JLabel("Only show files which pass");
      JLabel label2 = new JLabel("regular-expression filter.");
      JPanel p = new JPanel();
      p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
      p.add(new JLabel(" "));
      JPanel extPanel = new JPanel();
      extPanel.setLayout(new BoxLayout(extPanel, BoxLayout.Y_AXIS));
      extPanel.add(qfp.extField);
      p.add(extPanel);
      p.add(new JLabel(" "));
      p.add(label1);
      p.add(label2);
      p.setBorder(BorderFactory.createEmptyBorder(0, 8, 0, 8));

      JOptionPane optionPane = new JOptionPane(null, JOptionPane.PLAIN_MESSAGE,
                                              JOptionPane.DEFAULT_OPTION);
      JDialog dialog = optionPane.createDialog(v, "Filter");
      dialog.getContentPane().add(BorderLayout.NORTH, p);
      dialog.pack();
      dialog.setVisible(true);

      qfp.actionPerformed(new ActionEvent(qfp.extField,
                                          ActionEvent.ACTION_PERFORMED, ""));
    }
  }

  static class SynchronizeAction implements ActionListener {
    public void actionPerformed(ActionEvent evt) {
      View v = QuickFilePlugin.getViewForComponent((Component)evt.getSource());
      QuickFilePanel qfp = QuickFilePlugin.getQuickFilePanelForView(v);
      if(qfp == null) return;

      Buffer b = v.getBuffer();
      int idx = b.getPath().lastIndexOf(b.getName());
      String path = b.getPath();
      if (idx != -1) {
          path = path.substring(0, idx);
      }

      // Log.log(Log.DEBUG, this, "Synchronize Path: [" + path + "]");
      File dir = new File(path);
      if(dir != null && dir.isDirectory()) {
		// QuickFile doesn't seem to like paths terminated with a File.separator
        String p = dir.getPath();
        if (p.endsWith(File.separator)) {
          p = p.substring(0, p.length() - 1);
        }
        qfp.dirField.setText(p);
        qfp.dirField.addCurrentToHistory();
        qfp.actionPerformed(new ActionEvent(qfp.dirField,
                                            ActionEvent.ACTION_PERFORMED, ""));
      }
    }
  }
}
