/*
 * QuickFilePlugin.java - Main class of the QuickFile plugin for jEdit
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import org.gjt.sp.jedit.*;
import org.gjt.sp.jedit.gui.*;
import org.gjt.sp.jedit.msg.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;

/**
 * A plugin for jEdit that allows easy opening and switching between
 * many files in a directory tree (such as a Java package hieararchy).
 *
 * @author Jason Ginchereau
 */
public class QuickFilePlugin extends EBPlugin {

  private static Hashtable actions;
  private static boolean openOnce = false;

  /**
   * Returns a QuickFile tool bar button action.
   */
  public static ActionListener getAction(String name)
  {
    return (ActionListener)actions.get(name);
  }

  /**
   * Called at start up time when the plugin is being loaded.
   */
  public void start() {
    jEdit.addAction(new ToggleVisibleAction());

    // these actions are used to implement the toolbar buttons in the
    // QuickFile panel
    actions = new Hashtable();
    actions.put("quickfile.choose-dir",new QuickFilePanel.ChooseDirAction());
    actions.put("quickfile.goto-dir",new QuickFilePanel.GotoDirAction());
    actions.put("quickfile.refresh",new QuickFilePanel.RefreshAction());
    actions.put("quickfile.synchronize",new QuickFilePanel.SynchronizeAction());
    actions.put("quickfile.filter",new QuickFilePanel.SetFilterAction());

    for(int i = 0; i < QuickFileOptionPane.viewStyles.length; i++)
    {
      String viewStyle = QuickFileOptionPane.viewStyles[i];
      actions.put("quickfile.view." + viewStyle,new QuickFilePanel.SetViewAction(viewStyle));
    }

    openOnce = jEdit.getBooleanProperty("quickfile.openatstart");
  }

  /**
   * Newly created views call this method of each plugin in turn.
   * The default implementation does nothing, but most plugins will
   * want to override it to add instances of <code>javax.swing.JMenu</code>
   * and <code>javax.swing.JMenuItem</code> to the appropriate vectors.
   * The menus and menu items can be created dynamically by the plugin,
   * but the preferred way is to use the methods of the
   * <code>GUIUtilities</code> class.
   */
  public void createMenuItems(View view, Vector menus, Vector menuItems) {
    JMenuItem menuItem = GUIUtilities
        .loadMenuItem(view, "quickfile.toggle-vis");
    if(menuItem != null) {
      menuItems.addElement(menuItem);
    }
  }

  /**
   * This allows plugins to add their own option pane to the
   * <code>OptionsDialog</code>.
   */
  public void createOptionPanes(OptionsDialog od) {
    od.addOptionPane(new QuickFileOptionPane());
  }

  public void handleMessage(EBMessage msg) {
    if(msg instanceof ViewUpdate) {
      ViewUpdate vu = (ViewUpdate) msg;
      View v = vu.getView();
      if(vu.getWhat() == ViewUpdate.CREATED) {
        viewCreated(v);
      }
      else if(vu.getWhat() == ViewUpdate.CLOSED) {
        viewClosed(v);
      }
    }
  }

  private void viewCreated(View v) {
    if(openOnce) {
      new ToggleVisibleAction().actionPerformed(
          new ActionEvent(v, ActionEvent.ACTION_PERFORMED,
                          "quickfile.toggle-vis"));
      openOnce = false;
    }
  }
  
  private void viewClosed(View v) {
    QuickFilePanel qfp = getQuickFilePanelForView(v);
    if(qfp != null) {
      panelViewMap.remove(qfp);
      viewPanelMap.remove(v);
      if(qfp.frame != null)
        qfp.frame.dispose();
    }
  }

  private void addQuickFilePanelToView(final View view) {
    QuickFilePanel qfp = new QuickFilePanel(view);

    final boolean left
        = !jEdit.getBooleanProperty("quickfile.side");

    if(jEdit.getBooleanProperty("quickfile.docked")) {
      JComponent comp = view.getSplitPane();
      if(comp == null)
        comp = view.getEditPane();
      Container parent = comp.getParent();
      JPanel panel = new JPanel(new BorderLayout());
      panel.add(BorderLayout.CENTER,comp);
      comp = panel;

      JSplitPane sp;

      if(left) {
        sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, qfp, comp);
        setResizeWeight(sp, 0.0);
        parent.add(BorderLayout.CENTER, sp);
        try {
          int div = Integer.parseInt(jEdit.getProperty("quickfile.divider"));
          if(div >= 100)
            sp.setDividerLocation(div);
        } catch(NumberFormatException nfex) { }
      }
      else {
        sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, comp, qfp);
        setResizeWeight(sp, 1.0);
        parent.add(BorderLayout.CENTER, sp);
        try {
          int div = Integer.parseInt(jEdit.getProperty("quickfile.divider"));
          if(div >= 100)
            sp.setDividerLocation(view.getSize().width-sp.getDividerSize()-div);
        } catch(NumberFormatException nfex) { }
      }
      sp.setOneTouchExpandable(true);
      sp.setBorder(null);
      view.getRootPane().revalidate();

      qfp.addComponentListener(new ComponentAdapter() {
        public void componentResized(ComponentEvent evt) {
          JSplitPane splitter
              = getSplitterForQuickFilePanel((QuickFilePanel)evt.getSource());
          View v2 = getViewForQuickFilePanel((QuickFilePanel)evt.getSource());
          int div = splitter.getDividerLocation();
          if(!left) div = v2.getSize().width - splitter.getDividerSize() - div;
          jEdit.setProperty("quickfile.divider", "" + div);
        }
      });

      viewPanelMap.put(view, qfp);
      panelViewMap.put(qfp, view);
    }
    else {
      class Frame extends JFrame {
        public Frame(QuickFilePanel p) {
          super("QuickFile");
          getContentPane().setLayout(new BorderLayout());
          getContentPane().add(p);
	  p.frame = this;
          enableEvents(AWTEvent.KEY_EVENT_MASK);
        }
        // Catching all key events in the frame
        // requires some non-conventional event-handling.
        protected void processKeyEvent(KeyEvent evt) {
          if(evt.getKeyCode() == KeyEvent.VK_ESCAPE) setVisible(false);
          else super.processKeyEvent(evt);
        }
      }

      Frame f = new Frame(qfp);
      f.pack();
      Rectangle vr = view.getBounds();
      
      if(openOnce) {
        f.setBounds(QuickFileOptionPane.getIntegerProperty("quickfile.x", 0),
                    QuickFileOptionPane.getIntegerProperty("quickfile.y", 0),
                    QuickFileOptionPane.getIntegerProperty("quickfile.width",
                        f.getSize().width),
                    QuickFileOptionPane.getIntegerProperty("quickfile.height",
                        Math.max(f.getSize().height, vr.height * 3/4)));
      }
      else {
        vr.x = QuickFileOptionPane.getIntegerProperty("view.x", vr.x);
        vr.y = QuickFileOptionPane.getIntegerProperty("view.y", vr.y);
        vr.width = QuickFileOptionPane.getIntegerProperty("view.width", vr.width);
        vr.height = QuickFileOptionPane.getIntegerProperty("view.height", vr.height);
        if(left) {
          f.setLocation(vr.x, vr.y);
        }
        else {
          f.setLocation(vr.x + vr.width - f.getSize().width, vr.y);
        }

        Dimension fSize = f.getSize();
        if(fSize.height < vr.height * 3/4) {
          fSize.height = vr.height * 3/4;
          f.setSize(fSize);
        }
      }

      f.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
      f.addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent wevt) {
          EditAction action = jEdit.getAction("quickfile.toggle-vis");
          ActionEvent evt = new ActionEvent(view,
              ActionEvent.ACTION_PERFORMED, null);
          action.actionPerformed(evt);
        }
      });
      
      f.addComponentListener(new ComponentAdapter() {
        public void componentMoved(ComponentEvent evt) {
          Component frame = (Component) evt.getSource();
          Point fLocation = frame.getLocation();
          jEdit.setProperty("quickfile.x", String.valueOf(fLocation.x));
          jEdit.setProperty("quickfile.y", String.valueOf(fLocation.y));
        }
        public void componentResized(ComponentEvent evt) {
          Component frame = (Component) evt.getSource();
          Dimension fSize = frame.getSize();
          jEdit.setProperty("quickfile.width", String.valueOf(fSize.width));
          jEdit.setProperty("quickfile.height", String.valueOf(fSize.height));          
        }
      });

      f.setVisible(true);

      viewPanelMap.put(view, qfp);
      panelViewMap.put(qfp, view);
    }
  }

  static void setResizeWeight(JSplitPane sp, double weight) {
    // This method is actually only available with JDK 1.3 beta or later.
    // Use reflection so this method has no effect on earlier versions.
    final Class[] paramClasses = new Class[] { Double.TYPE };
    try {
      JSplitPane.class.getMethod("setResizeWeight", paramClasses)
                .invoke(sp, new Object[] { new Double(weight) });
    } catch(Exception ex) { }
  }

  public class ToggleVisibleAction extends EditAction {
    private int defaultDividerSize = -1;

    public ToggleVisibleAction() {
      super("quickfile.toggle-vis");
    }

    public boolean isToggle() {
      return true;
    }

    public boolean isSelected(Component comp) {
      View v = getView(comp);
      QuickFilePanel qfp = getQuickFilePanelForView(v);
      if(qfp != null) {
        JSplitPane sp = getSplitterForQuickFilePanel(qfp);
        if(sp != null) {
          int value = sp.getDividerLocation();
          if(sp.getLeftComponent() instanceof QuickFilePanel) {
            //int min = sp.getMinimumDividerLocation();
            int min = 120;
            return value >= min;
          }
          else if(sp.getRightComponent() instanceof QuickFilePanel) {
            //int max = sp.getMaximumDividerLocation();
            int max = sp.getWidth() - 120;
            return value <= max;
          }
        }
        else {
          Window w = SwingUtilities.windowForComponent(qfp);
          if(!(w instanceof View)) {
            return w.isVisible();
          }
        }
      }
      return false;
    }

    public void actionPerformed(ActionEvent evt) {
      boolean vis = false;
      View v = getView(evt);
      QuickFilePanel qfp = getQuickFilePanelForView(v);
      if(qfp == null) {
        addQuickFilePanelToView(v);
        vis = true;
      }
      else {
        JSplitPane sp = getSplitterForQuickFilePanel(qfp);
        if(sp != null) {
          int value = sp.getDividerLocation();
          if(sp.getLeftComponent() instanceof QuickFilePanel) {
            //int min = sp.getMinimumDividerLocation();
            int min = 120;
            if(value < min) {
              if(defaultDividerSize > 0) sp.setDividerSize(defaultDividerSize);
              int lastLocation = sp.getLastDividerLocation();
              if(lastLocation >= min) sp.setDividerLocation(lastLocation);
              else sp.setDividerLocation(min);
              vis = true;
            }
            else {
              sp.setDividerLocation(0.0);
              if(sp.getDividerSize() > 0)
                defaultDividerSize = sp.getDividerSize();
              if(jEdit.getBooleanProperty("quickfile.hidebar"))
                sp.setDividerSize(0);
              vis = false;
            }
          }
          else if(sp.getRightComponent() instanceof QuickFilePanel) {
            //int max = sp.getMaximumDividerLocation();
            int max = sp.getWidth() - 120;
            if(value > max) {
              if(defaultDividerSize > 0) sp.setDividerSize(defaultDividerSize);
              int lastLocation = sp.getLastDividerLocation();
              if(lastLocation <= max && lastLocation > 0) {
                sp.setDividerLocation(lastLocation);
              }
              else {
                sp.setDividerLocation(max);
              }
              vis = true;
            }
            else {
              if(sp.getDividerSize() > 0)
                defaultDividerSize = sp.getDividerSize();
              if(jEdit.getBooleanProperty("quickfile.hidebar")) {
                sp.setDividerSize(0);
                sp.setDividerLocation(sp.getSize().width-2);
              }
              else {
                sp.setDividerLocation(1.0);
              }
              vis = false;
            }
          }
        }
        else {
          Window w = SwingUtilities.windowForComponent(qfp);
          if(!(w instanceof View)) {
            vis = !w.isVisible();
            w.setVisible(vis);
          }
        }
      }

      //AbstractButton mi = (AbstractButton) viewMenuItemMap.get(v);
      //mi.setSelected(vis);
    }
  }

  private static Hashtable viewPanelMap = new Hashtable();
  private static Hashtable panelViewMap = new Hashtable();

  public static JSplitPane getSplitterForQuickFilePanel(QuickFilePanel p) {
    if(p == null) return null;
    Container c = p.getParent();
    while(c != null && !(c instanceof JSplitPane)) c = c.getParent();
    return (JSplitPane)c;
  }

  public static View getViewForQuickFilePanel(QuickFilePanel p) {
    return p != null? (View)panelViewMap.get(p) : null;
  }

  public static QuickFilePanel getQuickFilePanelForView(View v) {
    return v != null? (QuickFilePanel)viewPanelMap.get(v) : null;
  }

  public static View getViewForComponent(Component c) {
    Window w = SwingUtilities.windowForComponent(c);
    if(w == null || w instanceof View) return (View)w;

    Component[] contents = ((JFrame)w).getContentPane().getComponents();
    for(int i = 0; i < contents.length; i++)
      if(contents[i] instanceof QuickFilePanel)
        return getViewForQuickFilePanel((QuickFilePanel)contents[i]);
    return null;
  }
}
