/*
 * QuickFilePopupMenu.java - provides popup actions for rename, del, etc.
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

import org.gjt.sp.jedit.*;

/**
 *
 * @author Jason Ginchereau
 */
public class QuickFilePopupMenu extends JPopupMenu implements ActionListener {

  protected File          file;
  protected QuickFileView qfView;

  public QuickFilePopupMenu(File file, QuickFileView qfView) {
    this.file = file;
    this.qfView = qfView;
    
    initActions();
  }

  private void initActions() {
    if(file.isDirectory()) {
      boolean dcchroot = jEdit.getBooleanProperty("quickfile.dc-chroot");
      add(createMenuItem("goto", dcchroot));
      add(createMenuItem("newdir", false));
      add(createMenuItem("rename", false));
      add(createMenuItem("delete", false));
    }
    else if(jEdit.getBuffer(
        MiscUtilities.constructPath(null, file.getPath())) == null) {
      // file is not currently opened in jEdit
      add(createMenuItem("open", true));
      add(createMenuItem("rename", false));
      add(createMenuItem("delete", false));
    }
    else {
      // file is currently opened in jEdit
      boolean dcclose = jEdit.getBooleanProperty("quickfile.dc-close");
      add(createMenuItem("close", dcclose));
    }
  }
  
  private JMenuItem createMenuItem(String name, boolean bold) {
    String label = jEdit.getProperty("quickfile.filemenu." + name + ".label");
    JMenuItem mi = new JMenuItem(label);
    Font f = mi.getFont();
    f = new Font(f.getName(), (bold ? Font.BOLD : Font.PLAIN), f.getSize());
    mi.setFont(f);
    mi.setActionCommand(name);
    mi.addActionListener(this);
    return mi;
  }
  
  public void actionPerformed(ActionEvent evt) {
    if(evt.getActionCommand().equals("open")) {
      Buffer buffer = jEdit.getBuffer(file.getPath());
      if(buffer == null) {
        jEdit.openFile(QuickFilePlugin.getViewForComponent(qfView),
                       null, file.getPath(), false, false);
      }
    }
    else if(evt.getActionCommand().equals("close")) {
      Buffer buffer = jEdit.getBuffer(file.getPath());
      if(buffer != null) {
        jEdit.closeBuffer(QuickFilePlugin.getViewForComponent(qfView),
                          buffer);
      }
    }
    else if(evt.getActionCommand().equals("goto")) {
      new QuickFilePanel.GotoDirAction().actionPerformed(
          new ActionEvent(qfView, ActionEvent.ACTION_PERFORMED, "goto"));
    }
    else if(evt.getActionCommand().equals("newdir")) {
      String msg = "Create new directory under '" + file + "' named:";
      String input = JOptionPane.showInputDialog(qfView, msg,
          "New Directory", JOptionPane.QUESTION_MESSAGE);
      if(input != null) {
        File newDir = new File(file, input);
        if(newDir.exists()) {
          msg = "Directory '" + input + "' already exists.";
          JOptionPane.showMessageDialog(qfView, msg,
              "Create Directory Error", JOptionPane.ERROR_MESSAGE);
        }
        else if(!newDir.mkdir()) {
          msg = "Create directory failed: unknown reason.";
          JOptionPane.showMessageDialog(qfView, msg,
              "Create Directory Error", JOptionPane.ERROR_MESSAGE);
        }
        else {
          qfView.refresh();
        }
      }
    }
    else if(evt.getActionCommand().equals("rename")) {
      String msg = "Rename " + (file.isDirectory()? "directory" : "file") +
                   " '" + file + "' to:";
      String input = JOptionPane.showInputDialog(qfView, msg,
          "Rename " + (file.isDirectory() ? "Directory" : "File"),
          JOptionPane.QUESTION_MESSAGE);
      if(input != null) {
        File newFile = new File(file.getParent(), input);
        if(newFile.exists()) {
          msg = "File '" + input + "' already exists.";
          JOptionPane.showMessageDialog(qfView, msg,
              "Rename File Error", JOptionPane.ERROR_MESSAGE);
        }
        else if(!file.renameTo(newFile)) {
          msg = "Rename failed: unknown reason.";
          JOptionPane.showMessageDialog(qfView, msg,
              "Rename File Error", JOptionPane.ERROR_MESSAGE);
        }
        else {
          qfView.refresh();
        }
      }
    }
    else if(evt.getActionCommand().equals("delete")) {
      String msg = "Delete " + (file.isDirectory()? "directory" : "file") +
                   " '" + file + "'?";
      int ok = JOptionPane.showConfirmDialog(qfView, msg,
          "Confirm " + (file.isDirectory() ? "Directory" : "File") + " Delete",
          JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
      if(ok == JOptionPane.OK_OPTION) {
        if(!file.delete()) {
          msg = "Delete failed: " + (file.isDirectory() ?
                "directory not empty." : "unknown reason.");
          JOptionPane.showMessageDialog(qfView, msg,
              (file.isDirectory() ? "Directory" : "File") + " Delete Error",
              JOptionPane.ERROR_MESSAGE);
        }
        else {
          qfView.refresh();
        }
      }
    }
  }
}
