/*
 * QuickFileTreeView.java - 
 * Copyright (C) 1999 Jason Ginchereau
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import org.gjt.sp.jedit.jEdit;

/**
 *
 * @author Jason Ginchereau
 */
public class QuickFileTreeView extends QuickFileView
                               implements TreeSelectionListener,
                                          MouseListener {

  protected JTree        tree;
  protected FileTreeNode treeRoot;
  private   JScrollPane  treeScroller;

  private static FileCellRenderer renderer = new FileCellRenderer();

  public QuickFileTreeView(File dir) {
    super(dir);

    updateSorting();

    String parent = null;
    if(jEdit.getBooleanProperty("quickfile.showparent"))
      parent = dir.getParent();
    if(parent != null) {
      FileTreeNode.Parent hiddenRoot = new FileTreeNode.Parent(
          new File(parent), dir, filter);
      treeRoot = (FileTreeNode)
          hiddenRoot.getChildAt(hiddenRoot.getChildCount()-1);
      tree = new HelpfulJTree(hiddenRoot);
    }
    else {
      treeRoot = new FileTreeNode(dir, filter);
      tree = new HelpfulJTree(treeRoot);
    }

    tree.setEditable(false);
    tree.setScrollsOnExpand(
        !jEdit.getBooleanProperty("quickfile.dc-chroot"));
    tree.setRootVisible(false);
    tree.setShowsRootHandles(true);
    tree.getSelectionModel()
        .setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
    tree.setCellRenderer(renderer);
    tree.addTreeSelectionListener(this);
    tree.addMouseListener(this);

    if(parent != null) {
      tree.expandRow(1);
      tree.setSelectionRow(1);
    }

    treeScroller = new JScrollPane(tree);

    setLayout(new BorderLayout());
    add(BorderLayout.CENTER, treeScroller);
  }

  private void updateSorting() {
    FileTreeNode.sort
        = jEdit.getBooleanProperty("quickfile.sort");
    FileTreeNode.caseSensitive
        = jEdit.getBooleanProperty("quickfile.sort.cs");
  }

  public File getSelectedDirectory() {
    File f = getSelectedFile();
    return (f != null && f.isDirectory())? f : null;
  }

  public File getSelectedFile() {
    TreePath path = tree.getSelectionPath();
    if(path == null) return null;
    FileTreeNode n = (FileTreeNode) path.getLastPathComponent();
    return n.getFile();
  }

  /**
   * Refreshes this view by checking for newly created or deleted files.
   * Also re-applies the filter.
   */
  public void refresh() {
    updateSorting();

    treeRoot.refresh(filter, (DefaultTreeModel)tree.getModel());
  }

  // TreeSelectionListener interface:
  public void valueChanged(TreeSelectionEvent evt) {
    if(evt.isAddedPath()) {
      FileTreeNode node = (FileTreeNode)evt.getPath().getLastPathComponent();
      fireFileSelected(node.getFile());
    }
    else fireFileSelected(null);
  }

  // MouseListener interface:
  public void mouseClicked(MouseEvent evt) {
    if((evt.getModifiers() & MouseEvent.BUTTON1_MASK) != 0 &&
       evt.getClickCount() == 2) {
      TreePath path = tree.getPathForLocation(evt.getX(), evt.getY());
      FileTreeNode node = path != null?
          (FileTreeNode)path.getLastPathComponent() : null;
      if(node != null) fireFileActivated(node.getFile());
    }
    else if(evt.isPopupTrigger()) doPopup(evt);
  }
  public void mousePressed(MouseEvent evt)  { 
    TreePath path = tree.getPathForLocation(evt.getX(), evt.getY());
    if(path != null && !path.equals(tree.getSelectionPath()))
      tree.setSelectionPath(path);
    if(evt.isPopupTrigger()) doPopup(evt);
  }
  public void mouseReleased(MouseEvent evt) {
    if(evt.isPopupTrigger()) doPopup(evt);
  }
  public void mouseEntered(MouseEvent evt)  { }
  public void mouseExited(MouseEvent evt)   { }
  
  private void doPopup(MouseEvent evt) {
    TreePath path = tree.getPathForLocation(evt.getX(), evt.getY());
    FileTreeNode node = path != null?
        (FileTreeNode)path.getLastPathComponent() : null;
    if(node != null)
      showFilePopup(node.getFile(), (Component) evt.getSource(), evt.getPoint());
  }
}
